<?php


namespace App\Domain\Account;

use App\Query;
use Exception;
use App\Domain\User\User;
use App\Domain\Count\Count;
use App\Routines\Auth as Routines;
use Illuminate\Support\Collection;
use Illuminate\Pagination\LengthAwarePaginator;

class AccountUsersQuery extends Query
{
    /**
     * total
     *
     * @return int
     * @throws Exception
     */
    public function total(): int
    {
        $query = Routines::SP_GET_ACCOUNT_USERS_COUNT;
        $response = $this->mapper->get($query, array_slice($this->parameters, 0, $this->argsCount($query)))
            ->mapInto(Count::class);

        if ($response->isOk()) {
            return $response->getData()->first()->total ?? 0;
        }
        return 0;
    }

    /**
     * items
     *
     * @param int $page
     * @param int $perPage
     * @return Collection
     * @throws Exception
     */
    public function items(int $page = 1, int $perPage = 15): Collection
    {
        $this->parameters[] = $this->offset($page, $perPage);
        $this->parameters[] = $perPage;

        $result = $this->mapper->get(Routines::SP_GET_ACCOUNT_USERS, $this->parameters)
            ->mapInto(User::class);

        return $result->getData();
    }

    /**
     * paginate
     *
     * @param int $page
     * @param int $perPage
     * @return LengthAwarePaginator
     * @throws Exception
     */
    public function paginate(int $page = 1, int $perPage = 15): LengthAwarePaginator
    {
        $total = $this->total();

        // Only make a second request if count is greater than zero
        $data = new Collection();
        if ($total > 0) {
            $data = $this->items($page, $perPage);
        }

        return new LengthAwarePaginator($data, $total, $perPage, $page);
    }
}
