<?php

namespace App\Domain\Account;

use App\Domain\Service\Service;
use App\BaseMapper;
use App\Query;
use App\QueryFactory;
use App\Response\Response;
use App\Routines\Auth as Routines;
use Exception;
use App\Exceptions\RoutineQueryException;

class Mapper extends BaseMapper
{
    public $modelClass = Account::class;

    /**
     * save
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function save(array $parameters): Response
    {
        return $this->set(Routines::SP_SET_ACCOUNT, $parameters)
            ->mapInto($this->modelClass);
    }

    /**
     * find
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function find(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_ACCOUNT, $parameters)
            ->mapInto($this->modelClass);
    }

    /**
     * users
     *
     * @return Query
     */
    public function users(): Query
    {
        return QueryFactory::create(AccountUsersQuery::class, $this);
    }

    /**
     * attachUser
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function attachUser(array $parameters): Response
    {
        return $this->set(Routines::SP_SET_USER_ACCOUNT, $parameters);
    }

    /**
     * detachUser
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function detachUser(array $parameters): Response
    {
        return $this->set(Routines::SP_DEL_USER_ACCOUNT, $parameters);
    }

    /**
     * services
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function services(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_SERVICES, $parameters)
            ->mapInto(Service::class);
    }

}
