<?php

namespace App\Domain\User;

use Exception;
use App\BaseMapper;
use App\Response\Response;
use App\Domain\Account\Account;
use App\Routines\Auth as Routines;
use App\Exceptions\RoutineQueryException;

class Mapper extends BaseMapper
{
    public $modelClass = User::class;

    /**
     * save
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function save(array $parameters): Response
    {
        return $this->set(Routines::SP_SET_USER, $parameters)
            ->mapInto($this->modelClass);
    }

    /**
     * find
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function find(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_USER, $parameters)
            ->mapInto($this->modelClass);
    }

    /**
     * updateRememberToken
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function updateRememberToken(array $parameters): Response
    {
        return $this->get(Routines::SP_SET_USER_REMEMBER_TOKEN, $parameters)
            ->mapInto($this->modelClass);
    }

    /**
     * accounts
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function accounts(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_USER_ACCOUNTS, $parameters)
            ->mapInto(Account::class);
    }

    /**
     * newPasswordReset
     *
     * Creates a new token on table users in column user_token
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function newPasswordReset(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_USER_ACCOUNTS, $parameters)
            ->mapInto(PasswordReset::class);
    }

    /**
     * findResetPasswordEmailAndToken
     *
     * @param array $parameters
     * @return Response
     * @throws RoutineQueryException|Exception
     */
    public function findResetPasswordEmailAndToken(array $parameters): Response
    {
        return $this->get(Routines::SP_GET_USER_ACCOUNTS, $parameters)
            ->mapInto(PasswordReset::class);
    }
}
